# Copyright (C) 2014 Yusuf Ozturk
# This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.
# This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
# You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

function ConvertTo-WWPN
{
<#
    .SYNOPSIS
     
        Converts WWPN Integer Array to WWPN Hex String
		
    .EXAMPLE
     
        ConvertTo-WWPN -WWPNArray (Get-WmiObject -Namespace "root\wmi" -Class MSFC_FCAdapterHBAAttributes).NodeWWN
		
    .NOTES
    
        Author: Yusuf Ozturk
        Website: http://www.yusufozturk.info
        Email: ysfozy[at]gmail.com		
#>

[CmdletBinding(SupportsShouldProcess = $true)]
param (
	
    [Parameter(
        Mandatory = $true,
        HelpMessage = 'WWPN Array')]
    $WWPNArray,
	
	[Parameter(
        Mandatory = $false,
        HelpMessage = 'Debug Mode')]
    [switch]$DebugMode = $false
)
	# Enable Debug Mode
	if ($DebugMode)
	{
		$DebugPreference = "Continue"
	}
	else
	{
		$ErrorActionPreference = "silentlycontinue"
	}

	# Clear Values
	$WWPN = $Null;
	$WWPNInt = $Null;
	$WWPNIntArray = $Null;
	$WWPNObjArray = $Null;
	$CheckWWPNArray = $Null;
	
	# Check WWPN Array
	$CheckWWPNArray = $WWPNArray -as [int[]]
	
	if ($CheckWWPNArray)
	{		
		# Get Length
		$Length = $WWPNArray.Length;
		
		# Create WWPN Object Array
		for ($WWPNInt = 0; $WWPNInt -lt $Length; $WWPNInt++)
		{
			[int[]]$WWPNIntArray = $WWPNIntArray + [int] $WWPNArray[$WWPNInt];
			[Object[]]$WWPNObjArray = $WWPNObjArray + "{0:X}" -f $WWPNIntArray[$WWPNInt];
		}
		
		# Create WWPN Address
		for($WWPNInt = 0; $WWPNInt -lt $Length; $WWPNInt++)
		{
			if ($WWPNObjArray[$WWPNInt].Length -eq 1)
			{
				$WWPN = $WWPN + "0" + $WWPNObjArray[$WWPNInt];
			}
			else 
			{
				$WWPN = $WWPN + $WWPNObjArray[$WWPNInt];
			}
			  
			if ($WWPNInt -lt $Length - 1)
			{
				$WWPN = $WWPN + ":"
			}
			
			if ($WWPN.Length -eq "23")
			{
				# Output WWPN
				[String]$WWPN;
			}
			elseif ($WWPN.Length -eq "24")
			{
				# Fix WWPN Value
				$WWPN = $WWPN.Substring(0,23)
				
				# Output WWPN
				[String]$WWPN;
				
				# Clear WWPN
				$WWPN = $Null;
			}
		}
	}
}

function ConvertTo-WWPNInt
{
<#
    .SYNOPSIS
     
        Converts WWPN Hex String to WWPN Integer Array
		
    .EXAMPLE
     
        ConvertTo-WWPNInt -WWPN 
		
    .NOTES
    
        Author: Yusuf Ozturk
        Website: http://www.yusufozturk.info
        Email: ysfozy[at]gmail.com		
#>

[CmdletBinding(SupportsShouldProcess = $true)]
param (
	
    [Parameter(
        Mandatory = $true,
        HelpMessage = 'WWPN')]
    $WWPN,
	
	[Parameter(
        Mandatory = $false,
        HelpMessage = 'Debug Mode')]
    [switch]$DebugMode = $false
)
	# Enable Debug Mode
	if ($DebugMode)
	{
		$DebugPreference = "Continue"
	}
	else
	{
		$ErrorActionPreference = "silentlycontinue"
	}

	# Clear Values
	$WWPNArray = $Null;
	$WWPNValue = $Null;
	$WWPNValues = $Null;
	
	# Get VM vHBA WWPN
	[String[]]$WWPNArray = $WWPN.Split(":")
	
	# Get Length
	$Length = $WWPNArray.Length;
		
	# Create WWPN Integer Array
	for ($WWPNInt = 0; $WWPNInt -lt $Length; $WWPNInt++)
	{
		$WWPNString = $WWPNArray[$WWPNInt]
		$WWPNValue = [System.Convert]::ToInt32($WWPNString,16)
		
		[int[]]$WWPNValues = $WWPNValues + [int]$WWPNValue;
	}

	# Output WWPN
	[int[]]$WWPNValues;
}

function ConvertTo-WWPNStrip
{
<#
    .SYNOPSIS
     
        Converts WWPN Hex String to WWPN Stripped Array
		
    .EXAMPLE
     
        ConvertTo-WWPNStrip -WWPN 
		
    .NOTES
    
        Author: Yusuf Ozturk
        Website: http://www.yusufozturk.info
        Email: ysfozy[at]gmail.com		
#>

[CmdletBinding(SupportsShouldProcess = $true)]
param (
	
    [Parameter(
        Mandatory = $true,
        HelpMessage = 'WWPN')]
    $WWPN,
	
	[Parameter(
        Mandatory = $false,
        HelpMessage = 'Debug Mode')]
    [switch]$DebugMode = $false
)
	# Enable Debug Mode
	if ($DebugMode)
	{
		$DebugPreference = "Continue"
	}
	else
	{
		$ErrorActionPreference = "silentlycontinue"
	}

	# Clear Values
	$WWPNArray = $Null;
	$WWPNValues = $Null;
	
	# Get VM vHBA WWPN
	[String[]]$WWPNArray = $WWPN.Split(":")
	
	# Get Length
	$Length = $WWPNArray.Length;
		
	# Create WWPN Integer Array
	for ($WWPNInt = 0; $WWPNInt -lt $Length; $WWPNInt++)
	{
		[String]$WWPNValues = $WWPNValues + $WWPNArray[$WWPNInt];
	}

	# Output WWPN
	[String]$WWPNValues;
}

function Remove-VirtualPortRegistry
{
<#
    .SYNOPSIS
     
        Converts WWPN Hex String to WWPN Stripped Array
		
    .EXAMPLE
     
        ConvertTo-WWPNStrip -WWPN 
		
    .NOTES
    
        Author: Yusuf Ozturk
        Website: http://www.yusufozturk.info
        Email: ysfozy[at]gmail.com		
#>

[CmdletBinding(SupportsShouldProcess = $true)]
param (
	
    [Parameter(
        Mandatory = $true,
        HelpMessage = 'WWPN')]
    $WWPN,
	
	[Parameter(
        Mandatory = $false,
        HelpMessage = 'Debug Mode')]
    [switch]$DebugMode = $false
)
	# Enable Debug Mode
	if ($DebugMode)
	{
		$DebugPreference = "Continue"
	}
	else
	{
		$ErrorActionPreference = "silentlycontinue"
	}

	# Set Values
	$Status = 1;
	$Found = 1;

	# Get WWPN Stripped
	$WWPNStripped = ConvertTo-WWPNStrip -WWPN $WWPN
	
	# Get WWPN Int Array
	$WWPNIntArray = ConvertTo-WWPNInt -WWPN $WWPN
	
	# Get QLogic Registry Path
	$Path = "System\CurrentControlSet\Services\ql2300\Parameters\Device";
	
	# Get Registry Value
	$Reg = [Microsoft.Win32.RegistryKey]::OpenRemoteBaseKey('LocalMachine', $ComputerName)
	$RegKey = $Reg.OpenSubKey($Path, $true)
	$Data = $RegKey.GetValue($WWPNStripped)

	if (!$Data)
	{
		# Set Status
		$Status = 0;
	}
	else
	{
		# Set Values
		$k = 0;

		for ($i = 0; $i -lt $Data.Length; $i = $i + 160)
		{
			# Set Found
			$Found = 1;
			
			# Clear Values
			$Entry = $Null;
			$Data = $Null;
			$NewData = $Null;
			
			
			for ($j = ($i + 8); $j -lt ($i + 16); $j++)
			{
			  [int[]]$Entry = $Entry + [int]$Data[$j];
			}
			
			for ($j = 0; $j -lt 8; $j++)
			{
			  if ($Entry[$j + ($k * 8)] -ne $WWPNIntArray[$j])
			  {
				# Set Found
				$Found = 0;
			  }
			}

			if ($Found -eq 0)
			{
			  for ($j = 0; $j -lt 160; $j++)
			  {
				[int[]]$NewData = $NewData + [int]$Data[($j + $i)];
			  }
			}

			$k++;
		}

		if ($NewData)
		{
			# Replace Registry Key
			$RegKey.SetValue($WWPNStripped, [byte[]]$NewData, [Microsoft.Win32.RegistryValueKind]::Binary)
			
			# Set Status
			$Status = 2;
		}
		else
		{
			# Delete Registry Key
			$RegKey.DeleteValue($WWPNStripped)
			
			# Set Status
			$Status = 3;
		}
	}
	
	# Output Status
	$Status;
}

function Get-FCVirtualPort
{
<#
    .SYNOPSIS
     
        Gets FC HBA Virtual Ports
		
    .EXAMPLE
     
        Get-FCVirtualPort
		
    .EXAMPLE
     
        Get-FCVirtualPort -ComputerName "host.domain.com"
		
    .EXAMPLE
     
        Get-FCVirtualPort -FCAdapterWWPN "50:06:0B:00:00:C2:62:1C"
		
    .EXAMPLE
     
        Get-FCVirtualPort -ComputerName "host.domain.com" -FCAdapterWWPN "50:06:0B:00:00:C2:62:1C"
		
    .NOTES
    
        Author: Yusuf Ozturk
        Website: http://www.yusufozturk.info
        Email: ysfozy[at]gmail.com
#>

[CmdletBinding(SupportsShouldProcess = $true)]
param (
	
    [Parameter(
        Mandatory = $false,
        HelpMessage = 'Computer Name')]
    [string]$ComputerName = "localhost",
	
    [Parameter(
        Mandatory = $false,
        HelpMessage = 'FC Adapter WWPN')]
    [string]$FCAdapterWWPN,
	
	[Parameter(
        Mandatory = $false,
        HelpMessage = 'Detailed Mode')]
    [switch]$Detailed = $false,
	
	[Parameter(
        Mandatory = $false,
        HelpMessage = 'Debug Mode')]
    [switch]$DebugMode = $false
)
	# Enable Debug Mode
	if ($DebugMode)
	{
		$DebugPreference = "Continue"
	}
	else
	{
		$ErrorActionPreference = "silentlycontinue"
	}

	# Clear Values
	$FCAdapters = $Null;
	$pHBAWWNN = $Null;
	$pHBAWWPN = $Null;
	$pHBAInstanceName = $Null;
	
	# Get Fibre Channel Adapters
	$FCAdapters = Get-WmiObject -ComputerName $ComputerName -Namespace "root\wmi" -Class "MSFC_FibrePortNPIVAttributes"
	
	if ($FCAdapters)
	{
		foreach ($FCAdapter in $FCAdapters)
		{
			# Get FC Adapter Port Information
			$pHBAWWNN = ConvertTo-WWPN -WWPNArray $FCAdapter.WWNN
			$pHBAWWPN = ConvertTo-WWPN -WWPNArray $FCAdapter.WWPN
			$pHBAInstanceName = $FCAdapter.InstanceName
			
			if ($FCAdapterWWPN)
			{
				if ($pHBAWWPN -eq $FCAdapterWWPN)
				{
					# Get Virtual Ports
					$FCVirtualPorts = $FCAdapter.VirtualPorts
					
					if ($FCVirtualPorts)
					{
						foreach ($FCVirtualPort in $FCVirtualPorts)
						{
							# Get Virtual Port Information
							$vHBAWWNN = ConvertTo-WWPN -WWPNArray $FCVirtualPort.WWNN
							$vHBAWWPN = ConvertTo-WWPN -WWPNArray $FCVirtualPort.WWPN
							
							# Clear Values
							$VMFibreChannel = $Null;
							$VMSanName = $Null;
							$VMName = $Null;
							$VMSanSet = $Null;
							$VMState = $Null;
							
							# Get VM vHBA WWPN
							$VMvHBAWWPN = $vHBAWWPN.Replace(":","")
							
							# Get Hyper-V Virtual Machine
							$VMFibreChannel = Get-VM -ComputerName $ComputerName | Select -ExpandProperty FibreChannelHostBusAdapters | Where { $_.WorldWidePortNameSetA -eq $VMvHBAWWPN }
							
							if ($VMFibreChannel) 
							{
								# Get VM vHBA Information
								$VMSanSet = "Set A"
								$VMSanName = $VMFibreChannel.SanName
								$VMName = $VMFibreChannel.VMName
								
								# Get VM State
								$VMState = (Get-VM -ComputerName $ComputerName -Name $VMName).State
							}
							else
							{
								# Get Hyper-V Virtual Machine
								$VMFibreChannel = Get-VM -ComputerName $ComputerName | Select -ExpandProperty FibreChannelHostBusAdapters | Where { $_.WorldWidePortNameSetB -eq $VMvHBAWWPN }
								
								if ($VMFibreChannel) 
								{
									# Get VM vHBA Information
									$VMSanSet = "Set B"
									$VMSanName = $VMFibreChannel.SanName
									$VMName = $VMFibreChannel.VMName
									
									# Get VM State
									$VMState = (Get-VM -ComputerName $ComputerName -Name $VMName).State
								}
							}
						
							# Output Results
							$Properties = New-Object Psobject
							$Properties | Add-Member Noteproperty ComputerName $ComputerName
							if ($VMName) 
							{ 
								$Properties | Add-Member Noteproperty VMName $VMName
								$Properties | Add-Member Noteproperty VMState $VMState
								$Properties | Add-Member Noteproperty VMSanName $VMSanName
								$Properties | Add-Member Noteproperty VMSanSet $VMSanSet
							}
							$Properties | Add-Member Noteproperty vHBAWWNN $vHBAWWNN
							$Properties | Add-Member Noteproperty vHBAWWPN $vHBAWWPN
							$Properties | Add-Member Noteproperty pHBAWWNN $pHBAWWNN
							$Properties | Add-Member Noteproperty pHBAWWPN $pHBAWWPN
							if ($Detailed) { $Properties | Add-Member Noteproperty InstanceName $pHBAInstanceName }
							Write-Output $Properties
						}
					}
				}
			}
			else
			{
				# Get Virtual Ports
				$FCVirtualPorts = $FCAdapter.VirtualPorts
				
				if ($FCVirtualPorts)
				{
					foreach ($FCVirtualPort in $FCVirtualPorts)
					{
						# Get Virtual Port Information
						$vHBAWWNN = ConvertTo-WWPN -WWPNArray $FCVirtualPort.WWNN
						$vHBAWWPN = ConvertTo-WWPN -WWPNArray $FCVirtualPort.WWPN
					
						# Clear Values
						$VMFibreChannel = $Null;
						$VMSanName = $Null;
						$VMName = $Null;
						$VMSanSet = $Null;
						$VMState = $Null;
						
						# Get VM vHBA WWPN
						$VMvHBAWWPN = $vHBAWWPN.Replace(":","")
						
						# Get Hyper-V Virtual Machine
						$VMFibreChannel = Get-VM -ComputerName $ComputerName | Select -ExpandProperty FibreChannelHostBusAdapters | Where { $_.WorldWidePortNameSetA -eq $VMvHBAWWPN }
						
						if ($VMFibreChannel) 
						{
							# Get VM vHBA Information
							$VMSanSet = "Set A"
							$VMSanName = $VMFibreChannel.SanName
							$VMName = $VMFibreChannel.VMName
							
							# Get VM State
							$VMState = (Get-VM -ComputerName $ComputerName -Name $VMName).State
						}
						else
						{
							# Get Hyper-V Virtual Machine
							$VMFibreChannel = Get-VM -ComputerName $ComputerName | Select -ExpandProperty FibreChannelHostBusAdapters | Where { $_.WorldWidePortNameSetB -eq $VMvHBAWWPN }
							
							if ($VMFibreChannel) 
							{
								# Get VM vHBA Information
								$VMSanSet = "Set B"
								$VMSanName = $VMFibreChannel.SanName
								$VMName = $VMFibreChannel.VMName
								
								# Get VM State
								$VMState = (Get-VM -ComputerName $ComputerName -Name $VMName).State
							}
						}
					
						# Output Results
						$Properties = New-Object Psobject
						$Properties | Add-Member Noteproperty ComputerName $ComputerName
						if ($VMName) 
						{ 
							$Properties | Add-Member Noteproperty VMName $VMName
							$Properties | Add-Member Noteproperty VMState $VMState
							$Properties | Add-Member Noteproperty VMSanName $VMSanName
							$Properties | Add-Member Noteproperty VMSanSet $VMSanSet
						}
						$Properties | Add-Member Noteproperty vHBAWWNN $vHBAWWNN
						$Properties | Add-Member Noteproperty vHBAWWPN $vHBAWWPN
						$Properties | Add-Member Noteproperty pHBAWWNN $pHBAWWNN
						$Properties | Add-Member Noteproperty pHBAWWPN $pHBAWWPN
						if ($Detailed) { $Properties | Add-Member Noteproperty InstanceName $pHBAInstanceName }
						Write-Output $Properties
					}
				}
			}
		}
	}
}

function Get-InactiveFCVirtualPort
{
<#
    .SYNOPSIS
     
        Gets Inactive FC HBA Virtual Ports
		
    .EXAMPLE
     
        Get-InactiveFCVirtualPort
		
    .EXAMPLE
     
        Get-InactiveFCVirtualPort -ComputerName "host.domain.com"

    .NOTES
    
        Author: Yusuf Ozturk
        Website: http://www.yusufozturk.info
        Email: ysfozy[at]gmail.com
#>

[CmdletBinding(SupportsShouldProcess = $true)]
param (
	
    [Parameter(
        Mandatory = $false,
        HelpMessage = 'Computer Name')]
    [string]$ComputerName = "localhost",
	
	[Parameter(
        Mandatory = $false,
        HelpMessage = 'Detailed Mode')]
    [switch]$Detailed = $false,
	
	[Parameter(
        Mandatory = $false,
        HelpMessage = 'Debug Mode')]
    [switch]$DebugMode = $false
)
	# Enable Debug Mode
	if ($DebugMode)
	{
		$DebugPreference = "Continue"
	}
	else
	{
		$ErrorActionPreference = "silentlycontinue"
	}
	
	# Check Hyper-V Module
	$CheckModule = (Get-Module -ListAvailable | Where {$_.Name -eq "Hyper-V"}).Name
	
	if ($CheckModule -eq "Hyper-V")
	{
		# Import Hyper-V Module
		Import-Module "Hyper-V"	
		
		# Check Hyper-V Host
		$CheckHyperVHost = Get-VMHost -ComputerName $ComputerName
		
		if ($CheckHyperVHost)
		{
			if ($Detailed)
			{
				# Get Inactive Virtual Ports
				$InactiveVirtualPorts = Get-FCVirtualPort -ComputerName $ComputerName -Detailed | Where {$_.VMState -ne "Running"}
			}
			else
			{
				# Get Inactive Virtual Ports
				$InactiveVirtualPorts = Get-FCVirtualPort -ComputerName $ComputerName | Where {$_.VMState -ne "Running"}
			}
			
			# Output Inactive Virtual Ports
			$InactiveVirtualPorts
		}
		else
		{
			if ($DebugMode)
			{
				Write-Warning "Please make sure $ComputerName is reachable and a compatible Hyper-V version role is enabled."
				Write-Warning "Aborting.."
			}
		}
	}
	else
	{
		if ($DebugMode)
		{
			Write-Warning "Hyper-V Module is not available on this machine."
			Write-Warning "Aborting.."
		}
	}
}

function Remove-InactiveFCVirtualPort
{
<#
    .SYNOPSIS
     
        Removes Inactive FC HBA Virtual Ports
		
    .EXAMPLE
     
        Remove-InactiveFCVirtualPort
		
    .EXAMPLE
     
        Remove-InactiveFCVirtualPort -ComputerName "host.domain.com"
		
    .NOTES
    
        Author: Yusuf Ozturk
        Website: http://www.yusufozturk.info
        Email: ysfozy[at]gmail.com
#>

[CmdletBinding(SupportsShouldProcess = $true)]
param (
	
    [Parameter(
        Mandatory = $false,
        HelpMessage = 'Computer Name')]
    [string]$ComputerName = "localhost",
	
	[Parameter(
        Mandatory = $false,
        HelpMessage = 'Debug Mode')]
    [switch]$DebugMode = $false
)
	# Enable Debug Mode
	if ($DebugMode)
	{
		$DebugPreference = "Continue"
	}
	else
	{
		$ErrorActionPreference = "silentlycontinue"
	}

	# Get Inactive Virtual Ports
	$InactiveVirtualPorts = Get-FCVirtualPort -ComputerName $ComputerName -Detailed | Where {$_.VMState -ne "Running"}
	
	if ($InactiveVirtualPorts)
	{
		foreach ($InactiveVirtualPort in $InactiveVirtualPorts)
		{
			# Clear Values
			$vHBAWWPN = $Null;
			$InstanceName = $Null;
			
			# Get NPIV Info
			$vHBAWWPN = $InactiveVirtualPort.vHBAWWPN
			$InstanceName = $InactiveVirtualPort.InstanceName
		
			# Get NPIV Methods
			$NPIVMethods = Get-WmiObject -ComputerName $ComputerName -Namespace "root\wmi" -Class "MSFC_FibrePortNPIVMethods"

			foreach ($NPIVMethod in $NPIVMethods)
			{
				if ($NPIVMethod.InstanceName -eq $InstanceName)
				{
					# Get WWPN Int Array
					$WWPNInt = ConvertTo-WWPNInt -WWPN $vHBAWWPN;
					
					# Remove Virtual Port
					$Status = $NPIVMethod.RemoveVirtualPort($WWPNInt);
					
					# Check Operation Status
					if ($Status.Status -eq 0)
					{
						# Remove Registry Value
						$Status = Remove-VirtualPortRegistry -WWPN $vHBAWWPN
						if ($Status -eq 0)
						{
							Write-Host "Inactive Virtual Ports removed sucessfully."
						}
						else
						{
							Write-Host "Inactive Virtual Ports removed sucessfully. Registry Status: $Status"
						}
					}
					else
					{
						$Error = $Status.Status;
						Write-Host "Removing Inactive Virtual Ports failed with error: $Error"						  
					}
				}
			}		
		}
	}
	else
	{
		Write-Host "No Inactive Virtual Port is exist on system."
	}
}